module NSolve

contains

	function Dihotomia(f,left,right,eps)

	implicit none

		real(kind=8)::Dihotomia
		real(kind=8),intent(in)::left,right
		real(kind=8),intent(in),optional::eps

	interface

	function f(x)

	implicit none

	real(kind=8)::f
	real(kind=8),intent(in)::x

	
	end function f

	end interface


	real(kind=8)::tol,x_a,x_b,x_c,y_a,y_b,y_c
	logical::isFinal

	if(present(eps)) then

	tol=eps
	else
	tol=1.0d-6
	end if
	
	x_a=left;  x_b=right
	y_a=f(x_a);  y_b=f(x_b)

	if (sign(1.0d0,y_a)*sign(1.0d0,y_b)<=0.0d0) then
	isFinal=(abs(x_a-x_b)<tol).AND.(f(0.5d0*(x_a+x_b))<tol)
	do while (.NOT.isFinal)
	x_c=x_a+0.5d0*(x_b-x_a)
	y_c=f(x_c)
	if (sign(1.0d0,y_a)*sign(1.0d0,y_c)<=0.0d0) then
	x_b=x_c;  y_b=y_c
	else
	x_a=x_c;  y_a=y_c
	end if
	isFinal=(abs(x_a-x_b)<tol).AND.(f(0.5d0*(x_a-x_b))<tol)
	end do
	Dihotomia=x_a+0.5d0*(x_b-x_a)
	else
	stop 'Wrong Root Localizing Interval'
	end if
	end function Dihotomia

	function Newton(f,appr,eps,deriv,maxIter)

	use NDeriv, only :D

	implicit none

	real(kind=8)::Newton
	real(kind=8),intent(in)::appr
	real(kind=8),intent(in),optional::eps
	integer(kind=4),intent(in),optional::maxIter

	optional::deriv

	interface

	function f(x)

	implicit none

	real(kind=8)::f
	real(kind=8),intent(in)::x

	end function f

	function deriv(x)

	implicit none

	real(kind=8)::deriv
	real(kind=8),intent(in)::x

	end function deriv


	end interface

	real(kind=8)::tol,old,res,der,error
	integer(kind=4)::iter,k

	if(present(eps)) then

	tol=eps
	else
	tol=1.0d-6
	end if

	if(present(maxIter)) then

	iter=MaxIter
	else
	iter=100
	end if

	k=0; old=appr
	do
	k=k+1
	if(present(deriv)) then
	der=deriv(old)
	else
	der=D(f,old,1.0d-4)
	end if
	res=old-f(old)/der
	error=abs(res-old)
	old=res
	if(k>iter) stop 'Error! To Much Steps in Newton!'
	if(error<tol)exit
	end do

	Newton=res

	end function Newton
	

	function Secant(f,appr,eps,maxIter)

	implicit none

		real(kind=8)::Secant
		real(kind=8),intent(in)::appr
		real(kind=8),intent(in),optional::eps
		integer(kind=4),intent(in),optional::maxIter

	interface

	function f(x)

	implicit none

	real(kind=8)::f
	real(kind=8),intent(in)::x

	
	end function f

	end interface

	real(kind=8)::tol,old,delta,res,error
	integer(kind=4)::iter,k

	if(present(eps)) then

	tol=eps
	else
	tol=1.0d-6
	end if

	if(present(maxIter)) then

	iter=MaxIter
	else
	iter=100
	end if

	k=0; old=appr; delta=1.0d-1*appr
	do
	k=k+1
	res=old-f(old)*delta/(f(old)-f(old-delta))
	error=abs(res-old)
	old=res
	if(k>iter) stop 'Error! To Much Steps in Secant!'
	if(error<tol)exit
	end do

	Secant=res

	end function Secant

end module NSolve









