module NDSolve

contains

	subroutine odeSolve(fun, xStart, xEnd, yStart, yEnd, tol, hMax, hMin)

	implicit none
	
	interface

		function fun(t,y)
			implicit none
			real(kind=8) :: fun
			real(kind=8), intent(in) :: t,y
		end function fun
	end interface


	real(kind=8), intent(in) :: xStart, xEnd, yStart, tol, hMax, hMin
	real(kind=8), intent(out) :: yEnd


	real(kind=8) :: t, w, h
	logical :: flag
	
	real(kind=8) :: k1, k2, k3, k4, k5, k6, r, delta
	real(kind=8), parameter :: c2 = 1.0d0/4.0d0
	real(kind=8), parameter :: c31 = 3.0d0/8.0d0, c32 = 3.0d0/32.0d0, & 
								c33 = 9.0d0/32.0d0
	real(kind=8), parameter :: c41 = 12.0d0/3.0d0, c42 = 1932.0d0/2197.0d0, &
								c43 = 7200.0d0/2197.0d0, c44 = 7296.0d0/2197.0d0
	real(kind=8), parameter :: c51 = 439.0d0/216.0d0, c52 = 8.0d0, &
								c53 = 3680.0d0/216.0d0, c54 = 845.0d0/4104.0d0
	real(kind=8), parameter :: c61 = 1.0d0/2.0d0, c62 = 8.0d0/27.0d0, &
								c63 = 2.0d0, c64 = 3544.0d0/2565.0d0, c65 = 1859.0d0/4104.0d0, c66 = 11.0d0/40.0d0
	real(kind=8), parameter :: cr1 = 360.0d0, cr2 = 128.0d0/4275.0d0, &
								cr3 = 2197.0d0/75240.0d0, cr4 = 50.0d0, cr5 = 2.0d0/55.0d0
	real(kind=8), parameter :: cw1 = 25.0d0/216.0d0, cw2 = 1408.0d0/2565.0d0, &
								cw3 = 2197.0d0/4104.0d0, cw4 = 5.0d0

	t = xStart; w = yStart; h=hMax; flag = .TRUE.
	do while (flag)
		k1 = h*fun(t,w)
		k2 = h*fun(t+c2*h, w+c2*k1)
		k3 = h*fun(t+c31*h, w+c32*k1+c33*k2)
		k4 = h*fun(t+c41*h, w+c42*k1-c43*k2+c44*k3)
		k5 = h*fun(t+h, w+c51*k1-c52*k2+c53*k3-c54*k4)
		k6 = h*fun(t+c61*h, w-c62*k1+c63*k2-c64*k3+c65*k4-c66*k5)
		r= abs(k1/cr1 - cr2*k3-cr3*k4+k5/cr4+cr5*k6)/h
		if (r <= tol) then
			t=t+h
			w = w+cw1*k1+cw2*k3+cw3*k4 - k5/cw4
		end if
		delta = 0.84d0*(tol/r)*(2.5d-1)
		if (delta <= 0.1d0) then
			h = 0.1*h
		else if (delta >= 4.0d0) then
			h = 4.0d0*h
		else
			h = delta*h
		end if
		if (h > hMax) h = hMax
		if (t >= xEnd) then
			flag = .FALSE.
		else if ( t + h > xEnd) then
			h = xEnd - t
		else if (h < hMax) then
			flag= .FALSE.
			stop 'minimum h is exceeded'
		end if
	end do

	yEnd=w

	end subroutine odeSolve

end module NDSolve




